--echo ===============
--echo Fetch Records
--echo ===============

SELECT col_7,col_8,col_9 FROM tab5
WHERE col_2='bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb'
LIMIT 1;
